/**
 * Klasse Knoten zur Verwaltung von Usern als Datenelement 
 * innerhalb eines geordneten Binrbaums.
 * (Implementierung ohne Entwurfsmuster Kompositum)
 * 
 * @author Peter Brichzin 
 * @version 23.5.24
 */
class Knoten 
{

    /** Referenz auf das Datenelement */
    private User daten;

    /** Referenz auf den linken und rechten Nachfolger */
    private Knoten linkerNachfolger;

    /**
     * Konstruktor fr Objekte der Klasse Knoten ohne linken und rechten Nachfolger, 
     * d.h. die entsprechenden Referenzattribute haben den "Wert" null.
     * @param datenNeu Referenz auf das Datenelement, das vom Knoten verwaltet wird.
     */
    Knoten(User datenNeu) 
    {
        daten = datenNeu;
        linkerNachfolger = null;
    }

    /**
     * Fgt einen Usern als Datenelement, falls sein Schlssel noch nicht vorhanden ist,  
     * in die bestehende Datenstruktur ein. D.h. die Eigenschaft geordneter Binrbaum 
     * bleibt erhalten.
     * @param datenNeu einzufgender User als Datenelement 
     */
    void Einfgen(User datenNeu) 
    {
        // Falls es ein Datenelement mit dem gleichen Schlssel schon gibt: Einfgen abbrechen.
        if(daten.IstGleich(datenNeu)) 
        {
            System.out.println("Die Daten existieren schon!");
        }
        // Abfrage, ob im rechten oder linken Teilbaum eingefgt werden muss
        else
        {
            if(daten.IstGrerAls(datenNeu)) 
            {
                // wenn es einen linken Nachfolger gibt, 
                // dannn wird das Datenelement im linkem Teilbaum eingefgt,
                // sonst wird ein Knoten mit einer Referenz auf das Datenelement erzeugt und als linker Nachfolger zugewiesen
                if(linkerNachfolger != null) 
                {
                    linkerNachfolger.Einfgen(datenNeu);
                }
                else 
                {
                    linkerNachfolger = new Knoten(datenNeu);
                }
            }
            else 
            {
                // wenn es einen rechten Nachfolger gibt, 
                // dannn wird das Datenelement im rechten Teilbaum eingefgt,
                // sonst wird ein Knoten mit einer Referenz auf das Datenelement erzeugt und als rechter Nachfolger zugewiesen
                if(rechterNachfolger != null) 
                {
                    rechterNachfolger.Einfgen(datenNeu);
                }
                // sonst Datenelement im rechten Teilbaum einfgen
                else 
                {
                    rechterNachfolger = new Knoten(datenNeu);
                }
            }
        }
    }

    /**
     * Geben-Methode zu den Daten 
     * @return Referenz auf die Daten
     */
    User DatenGeben() 
    {
        return daten;
    }

    /**
     * Geben-Methode zum linken Nachfolger
     * @return Referenz auf LinkerNachfolger
     */
    Knoten LinkerNachfolgerGeben() 
    {
        return linkerNachfolger;
    }

    /**
     * Geben-Methode zum rechten Nachfolger
     * @return Referenz auf RechterNachfolger
     */
    Knoten RechterNachfolgerGeben() 
    {
        return rechterNachfolger;
    }
}
